# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for https://mangaread.org/"""

from .common import ChapterExtractor, MangaExtractor
from .. import text, exception
import re


class MangareadBase():
    """Base class for Mangaread extractors"""
    category = "mangaread"
    root = "https://www.mangaread.org"

    @staticmethod
    def parse_chapter_string(chapter_string, data):
        match = re.match(
            r"(?:(.+)\s*-\s*)?[Cc]hapter\s*(\d+)(\.\d+)?(?:\s*-\s*(.+))?",
            text.unescape(chapter_string).strip())
        manga, chapter, minor, title = match.groups()
        manga = manga.strip() if manga else ""
        data["manga"] = data.pop("manga", manga)
        data["chapter"] = text.parse_int(chapter)
        data["chapter_minor"] = minor or ""
        data["title"] = title or ""
        data["lang"] = "en"
        data["language"] = "English"


class MangareadChapterExtractor(MangareadBase, ChapterExtractor):
    """Extractor for manga-chapters from mangaread.org"""
    pattern = (r"(?:https?://)?(?:www\.)?mangaread\.org"
               r"(/manga/[^/?#]+/[^/?#]+)")
    test = (
        ("https://www.mangaread.org/manga/one-piece/chapter-1053-3/", {
            "pattern": (r"https://www\.mangaread\.org/wp-content/uploads"
                        r"/WP-manga/data/manga_[^/]+/[^/]+/[^.]+\.\w+"),
            "count": 11,
            "keyword": {
                "manga"   : "One Piece",
                "title"   : "",
                "chapter" : 1053,
                "chapter_minor": ".3",
                "tags"    : ["Oda Eiichiro"],
                "lang"    : "en",
                "language": "English",
            }
        }),
        ("https://www.mangaread.org/manga/one-piece/chapter-1000000/", {
            "exception": exception.NotFoundError,
        }),
        (("https://www.mangaread.org"
         "/manga/kanan-sama-wa-akumade-choroi/chapter-10/"), {
            "pattern": (r"https://www\.mangaread\.org/wp-content/uploads"
                        r"/WP-manga/data/manga_[^/]+/[^/]+/[^.]+\.\w+"),
            "count": 9,
            "keyword": {
                "manga"   : "Kanan-sama wa Akumade Choroi",
                "title"   : "",
                "chapter" : 10,
                "chapter_minor": "",
                "tags"    : list,
                "lang"    : "en",
                "language": "English",
            }
        }),
        # 'Chapter146.5'
        #        ^^ no whitespace
        ("https://www.mangaread.org/manga/above-all-gods/chapter146-5/", {
            "pattern": (r"https://www\.mangaread\.org/wp-content/uploads"
                        r"/WP-manga/data/manga_[^/]+/[^/]+/[^.]+\.\w+"),
            "count": 6,
            "keyword": {
                "manga"   : "Above All Gods",
                "title"   : "",
                "chapter" : 146,
                "chapter_minor": ".5",
                "tags"    : list,
                "lang"    : "en",
                "language": "English",
            }
        }),
    )

    def metadata(self, page):
        data = {"tags": list(text.extract_iter(page, "class>", "<"))}
        info = text.extr(page, '<h1 id="chapter-heading">', "</h1>")
        if not info:
            raise exception.NotFoundError("chapter")
        self.parse_chapter_string(info, data)
        return data

    def images(self, page):
        page = text.extr(
            page, '<div class="reading-content">', '<div class="entry-header')
        return [
            (url.strip(), None)
            for url in text.extract_iter(page, 'data-src="', '"')
        ]


class MangareadMangaExtractor(MangareadBase, MangaExtractor):
    """Extractor for manga from mangaread.org"""
    chapterclass = MangareadChapterExtractor
    pattern = r"(?:https?://)?(?:www\.)?mangaread\.org(/manga/[^/?#]+)/?$"
    test = (
        ("https://www.mangaread.org/manga/kanan-sama-wa-akumade-choroi", {
            "pattern": (r"https://www\.mangaread\.org/manga"
                        r"/kanan-sama-wa-akumade-choroi"
                        r"/chapter-\d+(-.+)?/"),
            "count"      : ">= 13",
            "keyword": {
                "manga"  : "Kanan-sama wa Akumade Choroi",
                "author" : ["nonco"],
                "artist" : ["nonco"],
                "type"   : "Manga",
                "genres" : ["Comedy", "Romance", "Shounen", "Supernatural"],
                "rating" : float,
                "release": 2022,
                "status" : "OnGoing",
                "lang"   : "en",
                "language"   : "English",
                "manga_alt"  : list,
                "description": str,
            }
        }),
        ("https://www.mangaread.org/manga/one-piece", {
            "pattern": (r"https://www\.mangaread\.org/manga"
                        r"/one-piece/chapter-\d+(-.+)?/"),
            "count"      : ">= 1066",
            "keyword": {
                "manga"  : "One Piece",
                "author" : ["Oda Eiichiro"],
                "artist" : ["Oda Eiichiro"],
                "type"   : "Manga",
                "genres" : list,
                "rating" : float,
                "release": 1997,
                "status" : "OnGoing",
                "lang"   : "en",
                "language"   : "English",
                "manga_alt"  : ["One Piece"],
                "description": str,
            }
        }),
        ("https://www.mangaread.org/manga/doesnotexist", {
            "exception": exception.NotFoundError,
        }),
    )

    def chapters(self, page):
        if 'class="error404' in page:
            raise exception.NotFoundError("manga")
        data = self.metadata(page)
        result = []
        for chapter in text.extract_iter(
                page, '<li class="wp-manga-chapter', "</li>"):
            url , pos = text.extract(chapter, '<a href="', '"')
            info, _ = text.extract(chapter, ">", "</a>", pos)
            self.parse_chapter_string(info, data)
            result.append((url, data.copy()))
        return result

    def metadata(self, page):
        extr = text.extract_from(text.extr(
            page, 'class="summary_content">', 'class="manga-action"'))
        return {
            "manga"      : text.extr(page, "<h1>", "</h1>").strip(),
            "description": text.unescape(text.remove_html(text.extract(
                page, ">", "</div>", page.index("summary__content"))[0])),
            "rating"     : text.parse_float(
                extr('total_votes">', "</span>").strip()),
            "manga_alt"  : text.remove_html(
                extr("Alternative </h5>\n</div>", "</div>")).split("; "),
            "author"     : list(text.extract_iter(
                extr('class="author-content">', "</div>"), '"tag">', "</a>")),
            "artist"     : list(text.extract_iter(
                extr('class="artist-content">', "</div>"), '"tag">', "</a>")),
            "genres"     : list(text.extract_iter(
                extr('class="genres-content">', "</div>"), '"tag">', "</a>")),
            "type"       : text.remove_html(
                extr("Type </h5>\n</div>", "</div>")),
            "release"    : text.parse_int(text.remove_html(
                extr("Release </h5>\n</div>", "</div>"))),
            "status"     : text.remove_html(
                extr("Status </h5>\n</div>", "</div>")),
        }



# -*- coding: utf-8 -*-

# Copyright 2016-2023 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for https://raw.senmanga.com/"""

from .common import ChapterExtractor
from .. import text


class SenmangaChapterExtractor(ChapterExtractor):
    """Extractor for manga chapters from raw.senmanga.com"""
    category = "senmanga"
    root = "https://raw.senmanga.com"
    pattern = r"(?:https?://)?raw\.senmanga\.com(/[^/?#]+/[^/?#]+)"
    test = (
        ("https://raw.senmanga.com/Bokura-wa-Minna-Kawaisou/37A/1", {
            "pattern": r"https://raw\.senmanga\.com/viewer"
                       r"/Bokura-wa-Minna-Kawaisou/37A/[12]",
            "url": "5f95140ff511d8497e2ec08fa7267c6bb231faec",
            "content": "556a16d5ca3441d7a5807b6b5ac06ec458a3e4ba",
            "keyword": {
                "chapter": "37A",
                "count": 2,
                "extension": "",
                "filename": "re:[12]",
                "lang": "ja",
                "language": "Japanese",
                "manga": "Bokura wa Minna Kawaisou",
                "page": int,
            },
        }),
        ("http://raw.senmanga.com/Love-Lab/2016-03/1", {
            "pattern": r"https://raw\.senmanga\.com/viewer"
                       r"/Love-Lab/2016-03/\d",
            "url": "8347b9f00c14b864dd3c19a1f5ae52adb2ef00de",
            "keyword": {
                "chapter": "2016-03",
                "count": 9,
                "extension": "",
                "filename": r"re:\d",
                "manga": "Renai Lab   恋愛ラボ",
            },
        }),
        ("https://raw.senmanga.com/akabane-honeko-no-bodyguard/1", {
            "pattern": r"https://i\d\.wp\.com/kumacdn.club/image-new-2/a"
                       r"/akabane-honeko-no-bodyguard/chapter-1"
                       r"/\d+-[0-9a-f]{13}\.jpg",
            "keyword": {
                "chapter": "1",
                "count": 65,
                "extension": "jpg",
                "filename": r"re:\d+-\w+",
                "manga": "Akabane Honeko no Bodyguard",
            },
        }),
    )

    def __init__(self, match):
        ChapterExtractor.__init__(self, match)
        self.session.headers["Referer"] = self.gallery_url

        # select "All pages" viewer
        self.session.cookies.set(
            "viewer", "1", domain="raw.senmanga.com")

    def metadata(self, page):
        title = text.extr(page, "<title>", "</title>")
        manga, _, chapter = title.partition(" - Chapter ")

        return {
            "manga"        : text.unescape(manga).replace("-", " "),
            "chapter"      : chapter.partition(" - Page ")[0],
            "chapter_minor": "",
            "lang"         : "ja",
            "language"     : "Japanese",
        }

    def images(self, page):
        return [
            (url, None)
            for url in text.extract_iter(
                page, '<img class="picture" src="', '"')
        ]



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for https://jpg.pet/"""

from .common import Extractor, Message
from .. import text

BASE_PATTERN = r"(?:https?://)?jpg\.(?:pet|fish(?:ing)?|church)"


class JpgfishExtractor(Extractor):
    """Base class for jpgfish extractors"""
    category = "jpgfish"
    root = "https://jpg.pet"
    directory_fmt = ("{category}", "{user}", "{album}",)
    archive_fmt = "{id}"

    def _pagination(self, url):
        while url:
            page = self.request(url).text

            for item in text.extract_iter(
                    page, '<div class="list-item-image ', 'image-container'):
                yield text.extract(item, '<a href="', '"')[0]

            url = text.extract(
                page, '<a data-pagination="next" href="', '" ><')[0]


class JpgfishImageExtractor(JpgfishExtractor):
    """Extractor for jpgfish Images"""
    subcategory = "image"
    pattern = BASE_PATTERN + r"/img/((?:[^/?#]+\.)?(\w+))"
    test = (
        ("https://jpg.pet/img/funnymeme.LecXGS", {
            "pattern": r"https://simp3\.jpg\.church/images/funnymeme\.jpg",
            "content": "098e5e9b17ad634358426e0ffd1c93871474d13c",
            "keyword": {
                "album": "",
                "extension": "jpg",
                "filename": "funnymeme",
                "id": "LecXGS",
                "url": "https://simp3.jpg.church/images/funnymeme.jpg",
                "user": "exearco",
            },
        }),
        ("https://jpg.church/img/auCruA", {
            "pattern": r"https://simp2\.jpg\.church/hannahowo_00457\.jpg",
            "keyword": {"album": "401-500"},
        }),
        ("https://jpg.fishing/img/funnymeme.LecXGS"),
        ("https://jpg.fish/img/funnymeme.LecXGS"),
        ("https://jpg.church/img/funnymeme.LecXGS"),
    )

    def __init__(self, match):
        JpgfishExtractor.__init__(self, match)
        self.path, self.image_id = match.groups()

    def items(self):
        url = "{}/img/{}".format(self.root, self.path)
        extr = text.extract_from(self.request(url).text)

        image = {
            "id"   : self.image_id,
            "url"  : extr('<meta property="og:image" content="', '"'),
            "album": text.extract(extr(
                "Added to <a", "/a>"), ">", "<")[0] or "",
            "user" : extr('username: "', '"'),
        }

        text.nameext_from_url(image["url"], image)
        yield Message.Directory, image
        yield Message.Url, image["url"], image


class JpgfishAlbumExtractor(JpgfishExtractor):
    """Extractor for jpgfish Albums"""
    subcategory = "album"
    pattern = BASE_PATTERN + r"/a(?:lbum)?/([^/?#]+)(/sub)?"
    test = (
        ("https://jpg.pet/album/CDilP/?sort=date_desc&page=1", {
            "count": 2,
        }),
        ("https://jpg.fishing/a/gunggingnsk.N9OOI", {
            "count": 114,
        }),
        ("https://jpg.fish/a/101-200.aNJ6A/", {
            "count": 100,
        }),
        ("https://jpg.church/a/hannahowo.aNTdH/sub", {
            "count": 606,
        }),
    )

    def __init__(self, match):
        JpgfishExtractor.__init__(self, match)
        self.album, self.sub_albums = match.groups()

    def items(self):
        url = "{}/a/{}".format(self.root, self.album)
        data = {"_extractor": JpgfishImageExtractor}

        if self.sub_albums:
            albums = self._pagination(url + "/sub")
        else:
            albums = (url,)

        for album in albums:
            for image in self._pagination(album):
                yield Message.Queue, image, data


class JpgfishUserExtractor(JpgfishExtractor):
    """Extractor for jpgfish Users"""
    subcategory = "user"
    pattern = BASE_PATTERN + r"/(?!img|a(?:lbum)?)([^/?#]+)(/albums)?"
    test = (
        ("https://jpg.pet/exearco", {
            "count": 3,
        }),
        ("https://jpg.church/exearco/albums", {
            "count": 1,
        }),
        ("https://jpg.fishing/exearco"),
        ("https://jpg.fish/exearco"),
        ("https://jpg.church/exearco"),
    )

    def __init__(self, match):
        JpgfishExtractor.__init__(self, match)
        self.user, self.albums = match.groups()

    def items(self):
        url = "{}/{}".format(self.root, self.user)

        if self.albums:
            url += "/albums"
            data = {"_extractor": JpgfishAlbumExtractor}
        else:
            data = {"_extractor": JpgfishImageExtractor}

        for url in self._pagination(url):
            yield Message.Queue, url, data
